SystemOrganization addCategory: #'Volunti-PublicBoard-Config'!
SystemOrganization addCategory: #'Volunti-PublicBoard-Model'!
SystemOrganization addCategory: #'Volunti-PublicBoard-View'!
SystemOrganization addCategory: #'Volunti-PublicBoard-Tests'!

TestCase subclass: #VOPublicBoardTest
	instanceVariableNames: 'thresholdPaillerPublicKey thresholdShares urn plainVotes publicBoard candidates results'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Volunti-PublicBoard-Tests'!

!VOPublicBoardTest methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 8/5/2012 14:13'!
candidates
	^ candidates
		ifNil: [ candidates := Array with: (VOChoice name: 'Nicolas') with: (VOChoice name: 'Amlie') with: (VOChoice name: 'Thomas') ]! !

!VOPublicBoardTest methodsFor: 'creation' stamp: 'NicolasVanderAuwera 8/5/2012 15:40'!
createUrn
	| ballot share encryptedVotes |
	urn := VOUrn new.
	plainVotes := OrderedCollection new.
	self numberOfVoters
		timesRepeat: [ 
			share := thresholdShares removeFirst.
			ballot := VOMBBallot numberOfPossibleVotes: self numberOfCandidates.
			ballot give: 1 additionalPointsTo: (1 to: self numberOfCandidates) atRandom.
			plainVotes add: ballot votes.
			encryptedVotes := thresholdPaillerPublicKey encrypt: ballot votes.
			ballot votes: (share blindArray: encryptedVotes with: thresholdPaillerPublicKey).
			urn addBallot: ballot ].
	results := plainVotes sum! !

!VOPublicBoardTest methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 8/5/2012 14:16'!
numberOfCandidates
	^ self candidates size! !

!VOPublicBoardTest methodsFor: 'running' stamp: 'NicolasVanderAuwera 8/5/2012 14:53'!
numberOfShares
	^ self numberOfVoters + self numberOfUnusedShares! !

!VOPublicBoardTest methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 8/5/2012 14:37'!
numberOfUnusedShares
	^ 20! !

!VOPublicBoardTest methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 8/5/2012 14:53'!
numberOfVoters
	^ 20! !

!VOPublicBoardTest methodsFor: 'running' stamp: 'NicolasVanderAuwera 8/6/2012 07:31'!
setUp
	| homomorphicAuthority |
	publicBoard := VOCompletePublicBoard new.
	homomorphicAuthority := VOHomomorphicAuthority new generateShares: self numberOfVoters.
	thresholdPaillerPublicKey := homomorphicAuthority thresholdPublicKey.
	thresholdShares := homomorphicAuthority shares asOrderedCollection.
	self createUrn.
	publicBoard := (VOPublicBoardManager current authorityWithId: 1000)
		urn: urn;
		candidates: candidates;
		thresholdPublicKey: thresholdPaillerPublicKey;
		paillierPrivateKey: homomorphicAuthority paillierPrivateKey;
		shares: thresholdShares;
		yourself! !

!VOPublicBoardTest methodsFor: 'tests' stamp: 'NicolasVanderAuwera 8/5/2012 15:42'!
testBlankBallots
	(publicBoard generateBlankBallots: 10) do: [ :ballot | self assert: ballot isBlank ]! !

!VOPublicBoardTest methodsFor: 'tests' stamp: 'NicolasVanderAuwera 8/5/2012 15:40'!
testResults
	self assert: thresholdShares size = self numberOfUnusedShares.
	publicBoard calculateResult.
	self assert: results = publicBoard results! !

TestCase subclass: #VOResultTest
	instanceVariableNames: 'result'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Volunti-PublicBoard-Tests'!

!VOResultTest methodsFor: 'running' stamp: 'NicolasVanderAuwera 12/11/2011 09:33'!
setUp
	result := VOResult id: 1 name: 'test'! !

!VOResultTest methodsFor: 'tests' stamp: 'NicolasVanderAuwera 12/11/2011 09:36'!
testAddToScore
	self assert: result score = 0.
	result addToScore: 2.
	self assert: result score = 2! !

WATask subclass: #VOCalculatorTask
	instanceVariableNames: 'modulo'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Volunti-PublicBoard-View'!

!VOCalculatorTask class methodsFor: 'instance creation' stamp: 'NicolasVanderAuwera 8/23/2012 14:47'!
modulo: aNumber
	^ self new modulo: aNumber! !

!VOCalculatorTask methodsFor: 'running' stamp: 'NicolasVanderAuwera 10/29/2012 14:45'!
go
	| calculator |
	[ 
	calculator := self call: (VOCalculatorComponent model: (VOCalculator modulo: modulo)).
	self inform: 'The result is: ' , calculator calculate greaseString ] repeat! !

!VOCalculatorTask methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 8/23/2012 14:48'!
modulo: aNumber
	modulo := aNumber! !

VOPage subclass: #VOAuthenticationPage
	instanceVariableNames: 'digipass authentication'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Volunti-PublicBoard-View'!

!VOAuthenticationPage methodsFor: 'hooks' stamp: 'NicolasVanderAuwera 8/22/2012 13:18'!
children
	^ OrderedCollection new
		addAll: super children;
		add: digipass;
		add: authentication;
		yourself! !

!VOAuthenticationPage methodsFor: 'initialization' stamp: 'NicolasVanderAuwera 8/22/2012 13:18'!
initialize
	super initialize.
	digipass := VODigipassTask new.
	authentication := VOZKAuthenticationTask new! !

!VOAuthenticationPage methodsFor: 'rendering' stamp: 'NicolasVanderAuwera 8/28/2012 09:03'!
renderAsideOn: html
	html div
		class: 'aside-block';
		with: [ 
					html image url: (VOFileLibrary urlOf: #digipassPng).
					html heading
						class: 'inline';
						level2;
						with: 'Digipass'.
					html horizontalRule.
					html render: digipass ]! !

!VOAuthenticationPage methodsFor: 'rendering' stamp: 'NicolasVanderAuwera 8/26/2012 10:13'!
renderContentOn: html
	html div
		class: 'twelve columns alpha';
		with: [ 
					html render: authentication.
					html anchor
						callback: [ self answer ];
						with: 'Back' ].
	html aside
		class: 'four columns omega';
		with: [ self renderAsideOn: html ]! !

!VOAuthenticationPage methodsFor: 'rendering' stamp: 'NicolasVanderAuwera 8/22/2012 13:25'!
renderMainContentOn: html
	html render: authentication! !

VOPage subclass: #VOCountingTreePage
	instanceVariableNames: 'randomNodeView countingTree publicBoard id randomNodeViews'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Volunti-PublicBoard-View'!

!VOCountingTreePage class methodsFor: 'instance creation' stamp: 'NicolasVanderAuwera 8/22/2012 14:15'!
countingTree: aCountingTree
	^ self new countingTree: aCountingTree! !

!VOCountingTreePage class methodsFor: 'instance creation' stamp: 'NicolasVanderAuwera 8/21/2012 15:52'!
countingTree: aCountingTree publicBoard: aPublicBoard
	^ self new
		countingTree: aCountingTree;
		publicBoard: aPublicBoard;
		yourself! !

!VOCountingTreePage methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 8/22/2012 13:14'!
aside
	^ #(#AuthenticationAnchor #Caption) asOrderedCollection! !

!VOCountingTreePage methodsFor: 'hooks' stamp: 'NicolasVanderAuwera 8/25/2012 08:37'!
children
	^ Array withAll: self randomNodeViews! !

!VOCountingTreePage methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 8/21/2012 15:33'!
countingTree: aCountingTree
	countingTree := aCountingTree! !

!VOCountingTreePage methodsFor: 'actions' stamp: 'NicolasVanderAuwera 8/21/2012 15:33'!
downloadCountingTree
	| file |
	file := WAFile new
		contentType: WAMimeType textPlain;
		contents: countingTree asXML;
		fileName: 'countingtree'.
	self requestContext
		respond: [ :response | 
			response
				contentType: 'text/xml';
				document: file rawContents greaseString;
				attachmentWithFileName: file fileName ]! !

!VOCountingTreePage methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 8/25/2012 09:27'!
javascriptCountingTree
	| js |
	js := WriteStream on: String new.
	js
		nextPutAll:
			'var render = function(r, n) {
				var label = r.text(0, 30, n.label); 
       			var set = r.set().push(
            			r.circle(25,50,50).attr({"fill": n.color, "stroke-width": 2, r : "20px"})).push(label);
        			return set;
    			};'.
	countingTree nodes
		do: [ :node | 
			js
				nextPutAll: 'g.addNode("';
				nextPutAll: node id greaseString;
				nextPutAll:
						'", {label : "' , node id greaseString , '", color : ' , (node color: self numberOfVotes) , ', render : render});' ].
	countingTree nodes
		do: [ :node | 
			node children
				do: [ :child | 
					js
						nextPutAll: 'g.addEdge("';
						nextPutAll: node id greaseString;
						nextPutAll: '","';
						nextPutAll: child id greaseString;
						nextPutAll: '", { directed : true });' ] ].
	^ js contents! !

!VOCountingTreePage methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 8/22/2012 10:20'!
numberOfVotes
	^ countingTree numberOfVotes! !

!VOCountingTreePage methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 8/21/2012 15:53'!
publicBoard: aPublicBoard
	publicBoard := aPublicBoard! !

!VOCountingTreePage methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 8/25/2012 08:37'!
randomNodeViews
	randomNodeViews isNil
		ifTrue: [ 
			randomNodeViews := OrderedCollection new
				add: (VOSimpleNodeView node: countingTree randomNode numberOfVoters: self numberOfVotes);
				add: (VOSimpleNodeView node: countingTree lowPercentageOfAccepts numberOfVoters: self numberOfVotes);
				add: (VOSimpleNodeView node: countingTree lowPercentageOfValidation numberOfVoters: self numberOfVotes);
				yourself ].
	^ randomNodeViews! !

!VOCountingTreePage methodsFor: 'rendering' stamp: 'NicolasVanderAuwera 8/25/2012 09:58'!
renderAsideOn: html
	html div
		class: 'aside-block';
		with: [ 
					self aside
						do: [ :symbol | self perform: ('render' , symbol capitalized , 'On:') asSymbol with: html ]
						separatedBy: [ html horizontalRule ] ]! !

!VOCountingTreePage methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 8/26/2012 10:17'!
renderAuthenticationAnchorOn: html
	self session isLoggedIn
		ifFalse: [ 
			html paragraph
				class: 'info';
				with: [ 
							html text: 'To validate nodes, please '.
							html anchor
								callback: [ 
											self call: VOAuthenticationPage new.
											self answer ];
								with: 'log in'.
							html text: '.' ] ]
		ifTrue: [ 
			html paragraph
				class: 'info';
				with: 'Welcome!! You can now validate node.'.
			html paragraph
				class: 'warning';
				with: [ 
							html text: 'National number: '.
							html strong: self session user nationalNumber ] ]! !

!VOCountingTreePage methodsFor: 'rendering' stamp: 'NicolasVanderAuwera 8/25/2012 09:48'!
renderCanvasOn: html
	html div id: 'canvas'.
	html
		paragraph: [ 
			html text: 'Not clear? '.
			html anchor
				class: 'js_anchor';
				onClick: 'redraw()';
				with: 'Redraw'.
			html text: ' or '.
			self renderDownloadXMLOn: html.
			html text: ' in XML format.'.
			self session isLoggedIn
				ifTrue: [ self renderGoToNodeOn: html ] ]! !

!VOCountingTreePage methodsFor: 'rendering' stamp: 'NicolasVanderAuwera 8/25/2012 09:57'!
renderCaptionOn: html
	html
		div: [ 
			html heading
				level: 3;
				with: 'Caption'.
			html image url: (VOFileLibrary urlOf: #captionPng).
			html div
				class: 'node-denied blank caption';
				with: [ html span: 'Blank vote' ] ]! !

!VOCountingTreePage methodsFor: 'rendering' stamp: 'NicolasVanderAuwera 8/21/2012 14:45'!
renderDownloadXMLOn: html
	html anchor
		callback: [ self downloadCountingTree ];
		with: 'download this counting tree'! !

!VOCountingTreePage methodsFor: 'rendering' stamp: 'NicolasVanderAuwera 8/22/2012 14:18'!
renderGoToNodeOn: html
	html paragraph: [ html text: 'Choose a node to validate it!!' ].
	html
		form: [ 
			html textInput
				class: 'inline';
				callback: [ :value | id := value greaseInteger ].
			html submitButton
				class: 'inline';
				callback: [ 
							self
								call:
									(VONodeEditorPage model: (countingTree nodeWithId: id) numberOfVotes: self numberOfVotes countingTree: countingTree) ];
				with: 'Go' ]! !

!VOCountingTreePage methodsFor: 'rendering' stamp: 'NicolasVanderAuwera 8/22/2012 13:38'!
renderMainContentOn: html
	self renderCanvasOn: html.
	html horizontalRule.
	self renderRandomNodeOn: html! !

!VOCountingTreePage methodsFor: 'rendering' stamp: 'NicolasVanderAuwera 8/25/2012 09:32'!
renderRandomNodeOn: html
	self session isLoggedIn
		ifTrue: [ 
			html heading
				level: 3;
				class: 'tit';
				with: 'Random nodes'.
			self randomNodeViews do: [ :view | html render: view ] ]! !

!VOCountingTreePage methodsFor: 'hooks' stamp: 'NicolasVanderAuwera 8/21/2012 14:33'!
script
	^ 'var redraw, g, renderer;

/* only do all this when document has finished loading (needed for RaphaelJS) */
$(function() {    
    g = new Graph();', self javascriptCountingTree , ' 

    /* layout the graph using the Spring layout implementation */
    var layouter = new Graph.Layout.Spring(g);

    /* draw the graph using the RaphaelJS draw implementation */
    renderer = new Graph.Renderer.Raphael(''canvas'', g, 600, 1200);

    redraw = function() {
        layouter.layout();
        renderer.draw();
    };
});'! !

!VOCountingTreePage methodsFor: 'hooks' stamp: 'NicolasVanderAuwera 8/21/2012 13:41'!
style
	^ super style	"^ '.node {
background-color: #E9EAEE;
-webkit-border-radius: 50px;
-moz-border-radius: 50px;
border-radius: 50px;
border: 1px #7F7D78 solid;
padding: 10px;
text-align: center;
margin: 5px 5px;
}
.block { position: absolute; }'"! !

!VOCountingTreePage methodsFor: 'updating' stamp: 'NicolasVanderAuwera 8/22/2012 14:16'!
updateRoot: aHtmlRoot
	super updateRoot: aHtmlRoot.
	aHtmlRoot title: 'Public Board - Counting Tree'.
	aHtmlRoot javascript url: (VOFileLibrary urlOf: #raphaelminJs).
	aHtmlRoot javascript url: (VOFileLibrary urlOf: #draculagraffleJs).
	aHtmlRoot javascript url: (VOFileLibrary urlOf: #draculagraphJs)! !

VOPage subclass: #VOMentalBoothResultPage
	instanceVariableNames: 'results resultsView candidates publicBoard'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Volunti-PublicBoard-View'!

!VOMentalBoothResultPage class methodsFor: 'instance creation' stamp: 'NicolasVanderAuwera 5/10/2012 09:23'!
publicBoard: aPublicBoard
	^ self new publicBoard: aPublicBoard! !

!VOMentalBoothResultPage methodsFor: 'hooks' stamp: 'NicolasVanderAuwera 8/21/2012 15:47'!
children
	^ Array with: self resultsView! !

!VOMentalBoothResultPage methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 5/10/2012 09:23'!
publicBoard: aPublicBoard
	publicBoard := aPublicBoard! !

!VOMentalBoothResultPage methodsFor: 'rendering' stamp: 'NicolasVanderAuwera 8/25/2012 10:23'!
renderBackOn: html
	html anchor
		class: 'button';
		callback: [ self call: (VOVotesPage ballots: self session protocol ballotsWithoutBlank) ];
		with: 'View ballots in a table'! !

!VOMentalBoothResultPage methodsFor: 'rendering' stamp: 'NicolasVanderAuwera 8/22/2012 14:15'!
renderCountingTreeAnchorOn: html
	html anchor
		class: 'button';
		callback: [ self call: (VOCountingTreePage countingTree: self session protocol countingTree) ];
		with: 'Check results with a counting tree'! !

!VOMentalBoothResultPage methodsFor: 'rendering' stamp: 'NicolasVanderAuwera 8/22/2012 13:27'!
renderMainContentOn: html
	html render: self resultsView.
	html div
		class: 'actions';
		with: [ 
					self renderCountingTreeAnchorOn: html.
					html text: ' or '.
					self renderBackOn: html ]! !

!VOMentalBoothResultPage methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 12/11/2012 07:38'!
resultsView
	resultsView
		ifNil: [ 
			resultsView := VOResultComponent
				results: self session protocol results
				candidates: self session protocol candidates
				numberOfVoters: self session protocol numberOfVoters ].
	^ resultsView! !

!VOMentalBoothResultPage methodsFor: 'as yet unclassified' stamp: 'NicolasVanderAuwera 8/25/2012 10:45'!
script
	^ '$(function() {  $("table").visualize({type: "pie", height: "300px", width: "420px"});});'! !

!VOMentalBoothResultPage methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 8/25/2012 10:26'!
title
	^ self session protocol name! !

!VOMentalBoothResultPage methodsFor: 'updating' stamp: 'NicolasVanderAuwera 8/25/2012 10:30'!
updateRoot: aHtmlRoot
	super updateRoot: aHtmlRoot.
	aHtmlRoot title: 'Results | ' , self title.
	aHtmlRoot stylesheet url: (VOFileLibrary urlOf: #visualizeCss).
	aHtmlRoot javascript url: (VOFileLibrary urlOf: #visualizejQueryJs)! !

VOPage subclass: #VONodePage
	instanceVariableNames: 'model numberOfVotes countingTree'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Volunti-PublicBoard-View'!

VONodePage subclass: #VONodeEditorPage
	instanceVariableNames: 'childViews calculator'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Volunti-PublicBoard-View'!

!VONodeEditorPage methodsFor: 'initialization' stamp: 'NicolasVanderAuwera 8/23/2012 14:47'!
calculator
	^ calculator ifNil: [ calculator := VOCalculatorTask modulo: countingTree modulo ]! !

!VONodeEditorPage methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 8/22/2012 14:18'!
childViews
	childViews isNil
		ifTrue: [ 
			childViews := self model children
				collect: [ :child | VONodePage model: child numberOfVotes: numberOfVotes countingTree: countingTree ] ].
	^ childViews! !

!VONodeEditorPage methodsFor: 'hooks' stamp: 'NicolasVanderAuwera 8/23/2012 14:47'!
children
	^ OrderedCollection new
		addAll: self childViews;
		add: self calculator;
		yourself! !

!VONodeEditorPage methodsFor: 'rendering' stamp: 'NicolasVanderAuwera 2/8/2012 14:59'!
renderBodyOn: html
	^ (self model hasAccepted: self session user)
		ifTrue: [ html strong: 'Accepted' ]
		ifFalse: [ 
			(self model hasDenied: self session user)
				ifTrue: [ html strong: 'Denied' ]
				ifFalse: [ self renderFormOn: html ] ]! !

!VONodeEditorPage methodsFor: 'rendering' stamp: 'NicolasVanderAuwera 8/23/2012 14:47'!
renderCalculatorOn: html
	html render: self calculator
					! !

!VONodeEditorPage methodsFor: 'rendering' stamp: 'NicolasVanderAuwera 2/9/2012 14:14'!
renderChildrenOn: html
	self childViews do: [ :childView | html render: childView ] separatedBy: [ html horizontalRule ]! !

!VONodeEditorPage methodsFor: 'rendering' stamp: 'NicolasVanderAuwera 8/22/2012 09:47'!
renderFormOn: html
	html div
		class: 'action';
		with: [ 
					html anchor
						class: 'accept';
						callback: [ self model addAccept: self session user name ];
						with: 'Accept'.
					html anchor
						class: 'deny';
						callback: [ self model addDeny: self session user name ];
						with: 'Deny' ].
	html break.
	html anchor
		class: 'button';
		callback: [ self show: '' ];
		with: 'Back to counting tree'! !

!VONodeEditorPage methodsFor: 'rendering' stamp: 'NicolasVanderAuwera 8/22/2012 14:19'!
renderMainContentOn: html
	html render: calculator.
	html paragraph: 'Number of voters: ' , numberOfVotes greaseString.
	super renderMainContentOn: html.
	html horizontalRule.
	self renderChildrenOn: html! !

!VONodePage class methodsFor: 'instance creation' stamp: 'NicolasVanderAuwera 8/22/2012 09:33'!
model: aNode numberOfVotes: aNumber
	^ self new
		model: aNode;
		numberOfVotes: aNumber;
		yourself! !

!VONodePage class methodsFor: 'instance creation' stamp: 'NicolasVanderAuwera 8/23/2012 14:46'!
model: aNode numberOfVotes: aNumber countingTree: aCountingTree
	^ (self model: aNode numberOfVotes: aNumber) countingTree: aCountingTree! !

!VONodePage methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 8/22/2012 09:48'!
countingTree: aCountingTree
	countingTree := aCountingTree! !

!VONodePage methodsFor: 'defaults' stamp: 'NicolasVanderAuwera 2/9/2012 14:10'!
defaultSpan
	^ 3! !

!VONodePage methodsFor: 'actions' stamp: 'NicolasVanderAuwera 8/22/2012 14:18'!
edit
	self call: (VONodeEditorPage model: self model)! !

!VONodePage methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 1/26/2012 17:02'!
model
	^ model! !

!VONodePage methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 1/26/2012 17:02'!
model: anObject
	model := anObject! !

!VONodePage methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 8/22/2012 09:33'!
numberOfVotes: aNumber
	numberOfVotes := aNumber! !

!VONodePage methodsFor: 'rendering' stamp: 'NicolasVanderAuwera 8/26/2012 10:14'!
renderBodyOn: html! !

!VONodePage methodsFor: 'rendering' stamp: 'NicolasVanderAuwera 8/25/2012 07:56'!
renderDetailsOn: html
	self renderStateOn: html.
	self renderValidationOn: html.
	self renderVotesOn: html! !

!VONodePage methodsFor: 'rendering' stamp: 'NicolasVanderAuwera 8/22/2012 14:18'!
renderIdOn: html
	html
		paragraph: [ 
			html text: 'id: '.
			html anchor
				callback: [ 
							self
								call:
									(VONodeEditorPage model: (countingTree nodeWithId: self model id) numberOfVotes: numberOfVotes countingTree: countingTree) ];
				with: self model id.
			model hasChildren
				ifFalse: [ html text: ' (ballot)' ] ]! !

!VONodePage methodsFor: 'rendering' stamp: 'NicolasVanderAuwera 8/22/2012 14:19'!
renderMainContentOn: html
	html div
		class: 'two';
		with: [ 
					self renderIdOn: html.
					self renderDetailsOn: html.
					self renderBodyOn: html ]! !

!VONodePage methodsFor: 'rendering' stamp: 'NicolasVanderAuwera 8/22/2012 10:04'!
renderStateOn: html
	html div
		class: 'block node-' , model state , '-' , (model validation: numberOfVotes) if: model isBlank not;
		class: 'block blank' if: model isBlank;
		with: [  ]! !

!VONodePage methodsFor: 'rendering' stamp: 'NicolasVanderAuwera 8/25/2012 07:56'!
renderValidationOn: html
	html
		paragraph: [ 
			html
				span: [ 
					html text: 'Percentage of Accepts: '.
					html strong: model percentageOfAccepted.
					html text: '%' ].
			html span
				style: 'float:right;';
				with: [ 
							html text: 'Percentage of Validation '.
							html strong: (model percentageOfVoters: numberOfVotes).
							html text: '%' ] ]! !

!VONodePage methodsFor: 'rendering' stamp: 'NicolasVanderAuwera 8/25/2012 08:18'!
renderVote: vote at: index on: html
	html label
		for: html nextId;
		with: 'Vote ' , index greaseString.
	html textArea
		id: html lastId;
		disabled: true;
		value: vote.
	model voterId isNil
		ifFalse: [ 
			html label
				for: html nextId;
				with: 'Signature ' , index greaseString.
			html textArea
				id: html lastId;
				disabled: true;
				value: (model signatures at: index) ]! !

!VONodePage methodsFor: 'rendering' stamp: 'NicolasVanderAuwera 8/25/2012 08:18'!
renderVotesOn: html
	html div
		class: 'votes container';
		with: [ 
					model voterId isNil
						ifFalse: [ 
							html
								paragraph: [ 
									html text: 'Votes of voter '.
									html strong: model voterId greaseString ] ].
					model votes
						keysAndValuesDo: [ :index :vote | 
							html div
								class: 'two column vote';
								with: [ self renderVote: vote at: index on: html ] ] ]! !

!VONodePage methodsFor: 'hooks' stamp: 'NicolasVanderAuwera 8/22/2012 09:21'!
style
	^ '.state {width: 45px; height: 30px; border-radius: 50px; border: 1px #DDD solid;}'! !

VOMainComponent subclass: #VOPublicBoradMainComponent
	instanceVariableNames: 'electionsList'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Volunti-PublicBoard-View'!

!VOPublicBoradMainComponent methodsFor: 'hooks' stamp: 'NicolasVanderAuwera 3/29/2012 13:49'!
children
	^ Array with: self electionsList! !

!VOPublicBoradMainComponent methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 8/22/2012 14:06'!
electionsList
	electionsList ifNil: [ electionsList := VOPBElectionsListPage elections: self agent elections ].
	^ electionsList! !

!VOPublicBoradMainComponent methodsFor: 'rendering' stamp: 'NicolasVanderAuwera 8/25/2012 10:17'!
renderMainContentOn: html
	super renderMainContentOn: html.
	html render: self electionsList! !

VOSessionAgent subclass: #VOPublicBoardSession
	instanceVariableNames: 'protocol'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Volunti-PublicBoard-Config'!

!VOPublicBoardSession methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 8/22/2012 13:19'!
protocol
	^ protocol! !

!VOPublicBoardSession methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 8/22/2012 13:19'!
protocol: anObject
	protocol := anObject! !

VOAgentComponent subclass: #VOPublicBoardAuthorityComponent
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Volunti-PublicBoard-View'!

!VOPublicBoardAuthorityComponent class methodsFor: 'class initialization' stamp: 'NicolasVanderAuwera 12/16/2012 09:59'!
initialize
	self register! !

!VOPublicBoardAuthorityComponent class methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 5/5/2012 07:27'!
sessionClass
	^ VOPublicBoardSession! !

!VOPublicBoardAuthorityComponent class methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 11/27/2011 13:26'!
url
	^ 'publicboard'! !

!VOPublicBoardAuthorityComponent methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 7/23/2012 07:40'!
agentClass
	^ VOPublicBoardManager! !

!VOPublicBoardAuthorityComponent methodsFor: 'private' stamp: 'NicolasVanderAuwera 11/25/2012 10:23'!
buildMainComponent
	^ VOPublicBoradMainComponent agent: self agent! !

!VOPublicBoardAuthorityComponent methodsFor: 'actions' stamp: 'NicolasVanderAuwera 8/22/2012 14:06'!
setHome
	content := VOPBElectionsListPage elections: self agent elections! !

!VOPublicBoardAuthorityComponent methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 11/27/2011 09:17'!
title
	^ 'Public Board'! !

WALabelledFormDialog subclass: #VOCalculatorComponent
	instanceVariableNames: 'model'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Volunti-PublicBoard-View'!

!VOCalculatorComponent class methodsFor: 'instance creation' stamp: 'NicolasVanderAuwera 8/23/2012 15:02'!
model: aCalculator
	^ self new model: aCalculator! !

!VOCalculatorComponent methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 8/6/2012 08:33'!
buttons
	^ #(#calculate)! !

!VOCalculatorComponent methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 8/6/2012 08:29'!
calculate
	self answer: model! !

!VOCalculatorComponent methodsFor: 'user interface' stamp: 'NicolasVanderAuwera 8/21/2012 10:34'!
labelForSelector: aSymbol
	aSymbol = #term1
		ifTrue: [ ^ 'Term 1' ].
	aSymbol = #term2
		ifTrue: [ ^ 'Term 2' ].
	aSymbol = #title
		ifTrue: [ ^ '' ].
	^ super labelForSelector: aSymbol! !

!VOCalculatorComponent methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 8/23/2012 15:02'!
model: aCalculator
	model := aCalculator! !

!VOCalculatorComponent methodsFor: 'rendering' stamp: 'NicolasVanderAuwera 8/21/2012 09:52'!
renderModuloOn: html
	html textInput
		disabled: true;
		value: model modulo! !

!VOCalculatorComponent methodsFor: 'rendering' stamp: 'NicolasVanderAuwera 8/6/2012 08:35'!
renderOperationOn: html
	html select
		list: #(#* #+);
		callback: [ :value | model operation: value asSymbol ]! !

!VOCalculatorComponent methodsFor: 'rendering' stamp: 'NicolasVanderAuwera 8/22/2012 08:58'!
renderTerm1On: html
	html textInput callback: [ :value | model term1: value greaseInteger ]! !

!VOCalculatorComponent methodsFor: 'rendering' stamp: 'NicolasVanderAuwera 8/22/2012 08:58'!
renderTerm2On: html
	html textInput callback: [ :value | model term2: value greaseInteger ]! !

!VOCalculatorComponent methodsFor: 'rendering' stamp: 'NicolasVanderAuwera 8/22/2012 09:03'!
renderTitleOn: html
	html image url: (VOFileLibrary urlOf: #devicePng).
	html heading
		level3;
		style: 'display: inline';
		with: 'Calculator'! !

!VOCalculatorComponent methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 8/21/2012 10:33'!
rows
	^ #(#title #term1 #term2 #modulo #operation)! !

!VOCalculatorComponent methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 8/22/2012 09:09'!
style
	^ '.dialog-form-field {clear: both; display: inline;}  .dialog-row {margin-left:10px;} .dialog-form-label {margin-right:5px; }'! !

VOSpecialUser subclass: #VOPublicBoardUser
	instanceVariableNames: 'password'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Volunti-PublicBoard-Model'!

!VOPublicBoardUser methodsFor: 'defaults' stamp: 'NicolasVanderAuwera 5/11/2012 07:33'!
defaultPasswordLength
	^ 8! !

!VOPublicBoardUser methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 5/11/2012 10:19'!
fields
	^ super fields , #(#password)! !

!VOPublicBoardUser methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 5/11/2012 10:20'!
password
	^ password! !

!VOPublicBoardUser methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 5/11/2012 07:33'!
password: aString
	password := aString! !

!VOPublicBoardUser methodsFor: 'validation' stamp: 'NicolasVanderAuwera 5/11/2012 07:34'!
validate
	super validate.
	self validatePassword! !

!VOPublicBoardUser methodsFor: 'validation' stamp: 'NicolasVanderAuwera 5/11/2012 07:34'!
validatePassword
	password size >= self defaultPasswordLength
		ifFalse: [ WAValidationNotification signal: 'Wrong information.' ]! !

WAComponent subclass: #VOSimpleNodeView
	instanceVariableNames: 'node numberOfVoters'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Volunti-PublicBoard-View'!

!VOSimpleNodeView class methodsFor: 'instance creation' stamp: 'NicolasVanderAuwera 8/19/2012 09:17'!
node: aNode numberOfVoters: aNumber
	^ self new
		node: aNode;
		numberOfVoters: aNumber;
		yourself! !

!VOSimpleNodeView methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 8/19/2012 09:12'!
node: aNode
	node := aNode! !

!VOSimpleNodeView methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 8/19/2012 09:18'!
numberOfVoters: aNumber
	numberOfVoters := aNumber! !

!VOSimpleNodeView methodsFor: 'rendering' stamp: 'NicolasVanderAuwera 8/26/2012 10:14'!
renderContentOn: html
	html div
		class: 'block node-' , node state , '-' , (node validation: numberOfVoters) if: node isBlank not;
		class: 'block blank' if: node isBlank;
		id: 'node' , node id greaseString;
		with: [ html strong: node id ]! !

WAComponent subclass: #VOVotesPage
	instanceVariableNames: 'report ballots'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Volunti-PublicBoard-View'!

!VOVotesPage class methodsFor: 'instance creation' stamp: 'NicolasVanderAuwera 8/25/2012 10:13'!
ballots: collection
	^ self new ballots: collection! !

!VOVotesPage methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 8/25/2012 10:14'!
ballots: collection
	ballots := collection! !

!VOVotesPage methodsFor: 'hooks' stamp: 'NicolasVanderAuwera 5/8/2012 15:36'!
children
	^ Array with: self report! !

!VOVotesPage methodsFor: 'rendering' stamp: 'NicolasVanderAuwera 5/8/2012 16:12'!
renderBackAnchorOn: html
	html anchor
		callback: [ self answer ];
		with: 'Back'! !

!VOVotesPage methodsFor: 'rendering' stamp: 'NicolasVanderAuwera 8/25/2012 10:21'!
renderContentOn: html
	html render: self report.
	self renderBackAnchorOn: html! !

!VOVotesPage methodsFor: 'initialization' stamp: 'NicolasVanderAuwera 8/25/2012 10:24'!
report
	^ report
		ifNil: [ 
			report := WATableReport new
				rows: ballots;
				columns:
						(OrderedCollection new
								add: (WAReportColumn selector: #voterId title: 'Voter ID');
								add: (WAReportColumn selector: #votes title: 'Votes');
								add: (WAReportColumn selector: #signatures title: 'Signatures');
								yourself);
				rowPeriod: 1;
				yourself ]! !

VOAuthority subclass: #VOPublicBoard
	instanceVariableNames: 'urn closed numberOfVoters candidates user'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Volunti-PublicBoard-Model'!

VOPublicBoard subclass: #VOCompletePublicBoard
	instanceVariableNames: 'countingTree shares paillierPrivateKey crypts thresholdPublicKey results'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Volunti-PublicBoard-Model'!

!VOCompletePublicBoard methodsFor: 'process' stamp: 'NicolasVanderAuwera 8/26/2012 10:08'!
authenticate: aUser
	"connects to the AA and receives the signed id"

	| anonymousAuth challenge client |
	user := aUser.
	anonymousAuth := self configurations at: VOAnonymousManager key.
	client := SocketStream openConnectionToHostNamed: anonymousAuth host port: anonymousAuth port.
	client
		sendCommand: #authenticate;
		sendCommand: id greaseString;
		sendObject: aUser.
	client nextLine asSymbol = #success
		ifTrue: [ challenge := client nextLine greaseInteger ].
	client close.
	^ challenge! !

!VOCompletePublicBoard methodsFor: 'process' stamp: 'NicolasVanderAuwera 10/31/2012 06:15'!
calculateResult
	numberOfVoters := urn tally.
	self encryptBlankBallots.
	results := (Array new: urn numberOfChoices) atAllPut: 1.
	urn ballots do: [ :ballot | results := results * ballot votes \\ thresholdPublicKey nsquared ].
	self decryptResult: results.
	countingTree := VOTreeCounting
		newFrom: urn ballots
		numberOfChildren: 2
		operation: #*
		modulo: thresholdPublicKey nsquared! !

!VOCompletePublicBoard methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 5/9/2012 07:11'!
countingTree
	^ countingTree! !

!VOCompletePublicBoard methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 5/7/2012 15:16'!
crypts
	^ crypts! !

!VOCompletePublicBoard methodsFor: 'process' stamp: 'NicolasVanderAuwera 5/10/2012 08:16'!
decryptResult: products
	results := self thresholdPublicKey unblindArray: products.
	results := self paillierPrivateKey decrypt: results with: self thresholdPublicKey! !

!VOCompletePublicBoard methodsFor: 'process' stamp: 'NicolasVanderAuwera 8/5/2012 15:39'!
encryptBlankBallots
	self shares
		with: (self generateBlankBallots: shares size)
		do: [ :share :bb | 
			bb votes: (share blindArray: (self thresholdPublicKey encrypt: bb votes) with: self thresholdPublicKey).
			urn addBallot: bb ]! !

!VOCompletePublicBoard methodsFor: 'process' stamp: 'NicolasVanderAuwera 8/5/2012 15:38'!
generateBlankBallots: number
	^ (1 to: number)
		collect: [ :i | ((Smalltalk at: urn ballotDesign) numberOfPossibleVotes: urn numberOfChoices) blank: true ]! !

!VOCompletePublicBoard methodsFor: 'initialization' stamp: 'NicolasVanderAuwera 5/7/2012 15:13'!
initialize
	super initialize.
	crypts := Dictionary new! !

!VOCompletePublicBoard methodsFor: 'testing' stamp: 'NicolasVanderAuwera 5/9/2012 07:10'!
isMentalBooth
	^ true! !

!VOCompletePublicBoard methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 5/9/2012 13:26'!
numberOfVoters
	^ numberOfVoters! !

!VOCompletePublicBoard methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 5/9/2012 12:54'!
paillierPrivateKey
	paillierPrivateKey ifNil: [ self requestPaillierPrivateKey ].
	^ paillierPrivateKey! !

!VOCompletePublicBoard methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 5/9/2012 12:57'!
paillierPrivateKey: aPaillierPrivateKey
	paillierPrivateKey := aPaillierPrivateKey! !

!VOCompletePublicBoard methodsFor: 'network' stamp: 'NicolasVanderAuwera 5/7/2012 15:17'!
receiveProofOn: stream
	| voterId cryptedVote |
	voterId := stream nextLine greaseInteger.
	cryptedVote := stream receiveObject.
	crypts at: voterId put: cryptedVote! !

!VOCompletePublicBoard methodsFor: 'process' stamp: 'NicolasVanderAuwera 5/9/2012 09:04'!
receiveUrn: aUrn
	urn := aUrn.
	numberOfVoters := urn tally.
	self calculateResult! !

!VOCompletePublicBoard methodsFor: 'network' stamp: 'NicolasVanderAuwera 7/23/2012 07:38'!
requestPaillierPrivateKey
	| client homomorphicAuth |
	homomorphicAuth := self configurations at: VOHomomorphicManager key.
	client := SocketStream openConnectionToHostNamed: homomorphicAuth host port: homomorphicAuth port.
	client
		sendCommand: 'paillierprivatekey';
		sendCommand: id greaseString.
	paillierPrivateKey := client receiveObject.
	client close! !

!VOCompletePublicBoard methodsFor: 'network' stamp: 'NicolasVanderAuwera 7/23/2012 07:38'!
requestThresholdPublicKey
	| client homomorphicAuth |
	homomorphicAuth := self configurations at: VOHomomorphicManager key.
	client := SocketStream openConnectionToHostNamed: homomorphicAuth host port: homomorphicAuth port.
	client
		sendCommand: 'thresholdpublickey';
		sendCommand: id greaseString.
	thresholdPublicKey := client receiveObject.
	client close! !

!VOCompletePublicBoard methodsFor: 'network' stamp: 'NicolasVanderAuwera 7/23/2012 07:39'!
requestUnusedShares
	| client pollingOffice |
	pollingOffice := self configurations at: VOPollingOfficeManager key.
	client := SocketStream openConnectionToHostNamed: pollingOffice host port: pollingOffice port.
	client
		sendCommand: 'unusedshares';
		sendCommand: id greaseString.
	shares := client receiveObject.
	client close! !

!VOCompletePublicBoard methodsFor: 'process' stamp: 'NicolasVanderAuwera 8/5/2012 15:18'!
results
	self assert: results notNil.
	^ results! !

!VOCompletePublicBoard methodsFor: 'network' stamp: 'NicolasVanderAuwera 8/26/2012 10:08'!
sendResponse: anInteger
	| anonymousAuth ok client |
	ok := true.
	anonymousAuth := self configurations at: VOAnonymousManager key.
	client := SocketStream openConnectionToHostNamed: anonymousAuth host port: anonymousAuth port.
	client
		sendCommand: #response;
		sendCommand: id greaseString;
		sendObject: user.
	client sendCommand: anInteger greaseString.
	client nextLine greaseString = 'error'
		ifTrue: [ ok := false ].
	client close.
	^ ok! !

!VOCompletePublicBoard methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 5/9/2012 10:44'!
shares
	shares ifNil: [ self requestUnusedShares ].
	^ shares! !

!VOCompletePublicBoard methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 5/9/2012 11:16'!
shares: aCollection
	shares := aCollection! !

!VOCompletePublicBoard methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 5/9/2012 12:43'!
thresholdPublicKey
	thresholdPublicKey ifNil: [ self requestThresholdPublicKey ].
	^ thresholdPublicKey! !

!VOCompletePublicBoard methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 5/9/2012 12:43'!
thresholdPublicKey: aPaillierPublicKey
	thresholdPublicKey := aPaillierPublicKey! !

!VOPublicBoard methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 8/25/2012 10:16'!
ballots
	^ urn ballots! !

!VOPublicBoard methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 8/25/2012 10:22'!
ballotsWithoutBlank
	^ urn ballots select: [ :ballot | ballot isBlank not ]! !

!VOPublicBoard methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 5/11/2012 10:33'!
candidates
	candidates ifNil: [ self requestCandidates ].
	^ candidates! !

!VOPublicBoard methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 8/5/2012 09:50'!
candidates: list
	candidates := list! !

!VOPublicBoard methodsFor: 'testing' stamp: 'NicolasVanderAuwera 5/10/2012 09:32'!
closed
	^ urn notNil! !

!VOPublicBoard methodsFor: 'initialization' stamp: 'NicolasVanderAuwera 3/29/2012 13:42'!
initialize
	super initialize.
	closed := false! !

!VOPublicBoard methodsFor: 'testing' stamp: 'NicolasVanderAuwera 5/9/2012 07:10'!
isMentalBooth
	^ false! !

!VOPublicBoard methodsFor: 'network' stamp: 'NicolasVanderAuwera 5/11/2012 10:37'!
requestCandidates
	| stream |
	stream := SocketStream openConnectionToHostNamed: NetNameResolver localHostName port: VOElectionManager current port.
	stream sendCommand: 'candidates'; sendCommand: id greaseString.
	candidates := stream receiveObject.
	stream close.
	! !

!VOPublicBoard methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 3/29/2012 13:42'!
urn: anUrn
	urn := anUrn! !

!VOPublicBoard methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 8/26/2012 10:11'!
user
	^ user! !

Object subclass: #VOCalculator
	instanceVariableNames: 'term1 term2 modulo operation'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Volunti-PublicBoard-Model'!

!VOCalculator class methodsFor: 'instance creation' stamp: 'NicolasVanderAuwera 8/21/2012 09:52'!
modulo: aNumber
	^ self new modulo: aNumber! !

!VOCalculator methodsFor: 'operation' stamp: 'NicolasVanderAuwera 8/22/2012 09:02'!
calculate
	^ (term1 perform: operation with: term2) \\ modulo! !

!VOCalculator methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 8/21/2012 09:50'!
modulo
	^ modulo! !

!VOCalculator methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 8/21/2012 09:50'!
modulo: aNumber
	modulo := aNumber! !

!VOCalculator methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 8/6/2012 08:24'!
operation: aSymbol
	operation := aSymbol! !

!VOCalculator methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 8/6/2012 08:24'!
term1: aNumber
	term1 := aNumber! !

!VOCalculator methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 8/6/2012 08:24'!
term2: aNumber
	term2 := aNumber! !

VOElectionsListComponent subclass: #VOPBElectionsListPage
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Volunti-PublicBoard-View'!

!VOPBElectionsListPage methodsFor: 'rendering' stamp: 'NicolasVanderAuwera 8/25/2012 10:15'!
renderMoreData: election on: html
	html
		tableData: [ 
			election opened
				ifTrue: [ 
					html anchor
						callback: [ self call: (VOVotesPage encryptedVotes: election crypts) ];
						with: 'Check my vote' ].
			(election opened and: [ election closed ])
				ifTrue: [ html text: ' | ' ].
			election closed
				ifTrue: [ 
					html anchor
						callback: [ self resultsFor: election ];
						with: 'Results' ] ]! !

!VOPBElectionsListPage methodsFor: 'rendering' stamp: 'NicolasVanderAuwera 3/29/2012 13:47'!
renderMoreDataHeadingOn: html
	html tableHeading: 'Action'! !

!VOPBElectionsListPage methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 8/22/2012 14:15'!
resultsFor: anElection
	self session protocol: anElection.
	self call: VOMentalBoothResultPage new! !

GRObject subclass: #VOElectionResults
	instanceVariableNames: 'results urn election'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Volunti-PublicBoard-Model'!

!VOElectionResults commentStamp: 'NicolasVanderAuwera 3/1/2012 09:50' prior: 0!
I contains an election and its associated urn. When these two instances are available, I calculate the results of the election. I am not use for Mental Booth election.!

!VOElectionResults methodsFor: 'comparing' stamp: 'NicolasVanderAuwera 12/19/2011 09:59'!
<=  someResults
	^ self electionId <= results electionId! !

!VOElectionResults methodsFor: 'evaluating' stamp: 'NicolasVanderAuwera 1/2/2012 11:00'!
buildResults
	self isAvailable
		ifFalse: [ self error: 'The election and the urn must be present' ].
	self initializeResults.
	urn ballots
		do: [ :ballot | 
			self verifyBallot: ballot.
			self calculate: ballot ]! !

!VOElectionResults methodsFor: 'evaluating' stamp: 'NicolasVanderAuwera 1/2/2012 11:00'!
calculate: aBallot
	| result |
	aBallot votes
		do: [ :vote | 
			result := self resultOf: vote id.
			result addToScore: vote score ]! !

!VOElectionResults methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 12/9/2011 08:51'!
election
	^ election! !

!VOElectionResults methodsFor: 'evaluating' stamp: 'NicolasVanderAuwera 12/11/2011 11:15'!
election: anElection
	election := anElection! !

!VOElectionResults methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 12/9/2011 09:16'!
electionId
	"if I exists, the election or the urn is not nil"

	urn notNil
		ifTrue: [ ^ urn id ]
		ifFalse: [ ^ election id ]! !

!VOElectionResults methodsFor: 'initialization' stamp: 'NicolasVanderAuwera 12/9/2011 09:13'!
initialize
	super initialize.
	urn := nil.
	election := nil! !

!VOElectionResults methodsFor: 'evaluating' stamp: 'NicolasVanderAuwera 3/1/2012 15:42'!
initializeResults
	results := (election choices collect: [ :choice | VOResult id: choice id name: choice name ]) asSortedCollection! !

!VOElectionResults methodsFor: 'testing' stamp: 'NicolasVanderAuwera 12/9/2011 09:48'!
isAvailable
	^ election notNil and: [ urn notNil ]! !

!VOElectionResults methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 12/9/2011 10:05'!
name
	^ election name! !

!VOElectionResults methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 12/9/2011 14:55'!
resultOf: anChoiceId
	^ self results detect: [ :each | each id = anChoiceId ] ifNone: [ nil ]! !

!VOElectionResults methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 12/20/2011 07:17'!
results
	^ results := self buildResults! !

!VOElectionResults methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 12/14/2011 15:10'!
size
	^ self results size! !

!VOElectionResults methodsFor: 'evaluating' stamp: 'NicolasVanderAuwera 12/9/2011 08:51'!
urn
	^ urn! !

!VOElectionResults methodsFor: 'evaluating' stamp: 'NicolasVanderAuwera 12/11/2011 11:15'!
urn: anUrn
	urn := anUrn! !

!VOElectionResults methodsFor: 'private' stamp: 'NicolasVanderAuwera 12/11/2011 11:30'!
verifyBallot: aBallot
	| ids |
	ids := election choices collect: [ :choice | choice id ].
	aBallot votes do: [ :vote | self assert: (ids includes: vote id) ]! !

!VOElectionResults methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 12/19/2011 15:32'!
voters
	^ urn ballots collect: [ :each | each voterId ]! !

GRObject subclass: #VOResult
	instanceVariableNames: 'id name score'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Volunti-PublicBoard-Model'!

!VOResult class methodsFor: 'instance creation' stamp: 'NicolasVanderAuwera 12/11/2011 09:28'!
id: anId name: aString
	^ self new
		id: anId;
		name: aString;
		yourself! !

!VOResult methodsFor: 'comparing' stamp: 'NicolasVanderAuwera 12/7/2011 09:16'!
<= aResult
	^ self score <= aResult score! !

!VOResult methodsFor: 'adding' stamp: 'NicolasVanderAuwera 2/29/2012 15:49'!
addToScore: anInteger
	self assert: anInteger isInteger.
	self assert: anInteger >= 0.
	self score: self score + anInteger.
	self assertInvariant! !

!VOResult methodsFor: 'testing' stamp: 'NicolasVanderAuwera 2/29/2012 15:49'!
assertInvariant
	self assert: self score >= 0! !

!VOResult methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 11/20/2011 14:09'!
id
	^ id! !

!VOResult methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 11/20/2011 14:09'!
id: anObject
	id := anObject! !

!VOResult methodsFor: 'initialization' stamp: 'NicolasVanderAuwera 11/26/2011 13:55'!
initialize
	super initialize.
	id := -1.
	name := ''.
	score := 0! !

!VOResult methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 11/20/2011 14:09'!
name
	^ name! !

!VOResult methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 11/20/2011 14:09'!
name: anObject
	name := anObject! !

!VOResult methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 12/11/2011 09:27'!
score
	^ score ifNil: [ score := 0 ]! !

!VOResult methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 12/11/2011 09:26'!
score: anInteger
	self assert: anInteger isInteger.
	self assert: anInteger >= 0.
	score := anInteger! !

VOManager subclass: #VOPublicBoardManager
	instanceVariableNames: 'collectionOfElectionResults countingTreeSample'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Volunti-PublicBoard-Model'!

!VOPublicBoardManager class methodsFor: 'x509' stamp: 'NicolasVanderAuwera 5/5/2012 13:41'!
organisationUnit
	^ 'Public Board'! !

!VOPublicBoardManager class methodsFor: 'network' stamp: 'NicolasVanderAuwera 12/1/2011 09:24'!
port
	^ 5678
	! !

!VOPublicBoardManager methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 8/19/2012 09:02'!
authorityClass
	^ VOPublicBoard! !

!VOPublicBoardManager methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 7/23/2012 09:08'!
authorityFor: aProtocol
	(aProtocol = 'mentalbooth' or: [ aProtocol = 'alternative' ])
		ifTrue: [ ^ VOCompletePublicBoard ].
	^ VOPublicBoard! !

!VOPublicBoardManager methodsFor: 'network' stamp: 'NicolasVanderAuwera 7/23/2012 07:48'!
closeElection
	authorities := self authorityWithId: stream nextLine greaseInteger.
	authorities closed.
	self addInLog: 'Election closed'! !

!VOPublicBoardManager methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 12/20/2011 07:37'!
collectionOfElectionResults
	^ collectionOfElectionResults ifNil: [ collectionOfElectionResults := OrderedCollection new ]! !

!VOPublicBoardManager methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 2/8/2012 15:11'!
countingTreeSample
	countingTreeSample isNil
		ifTrue: [ countingTreeSample := self sample ].
	^ countingTreeSample! !

!VOPublicBoardManager methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 7/23/2012 07:49'!
elections
	^ self authorities" select: [ :agent | agent isClosed ]"! !

!VOPublicBoardManager methodsFor: 'network' stamp: 'NicolasVanderAuwera 5/7/2012 15:14'!
handle: aSymbol
	"aSymbol = #ballot
		ifTrue: [ self handleBallot ]."

	aSymbol = #urn
		ifTrue: [ self receiveUrn ].
	aSymbol = #proof
		ifTrue: [ self receiveProof ].
	super handle: aSymbol! !

!VOPublicBoardManager methodsFor: 'initialization' stamp: 'NicolasVanderAuwera 12/14/2011 08:33'!
initialize
	super initialize.
	collectionOfElectionResults := OrderedCollection new! !

!VOPublicBoardManager methodsFor: 'network' stamp: 'NicolasVanderAuwera 7/27/2012 08:00'!
receiveProof
	| subAgent |
	subAgent := self authorityWithId: self idOnStream.
	subAgent receiveProofOn: stream! !

!VOPublicBoardManager methodsFor: 'network' stamp: 'NicolasVanderAuwera 7/27/2012 08:00'!
receiveUrn
	self authorityOnStream receiveUrn: stream receiveObject! !

!VOPublicBoardManager methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 12/9/2011 09:40'!
resultsOf: anElectionId
	^ collectionOfElectionResults detect: [ :each | each electionId = anElectionId ] ifNone: [ nil ]! !

!VOPublicBoardManager methodsFor: 'sample' stamp: 'NicolasVanderAuwera 10/31/2012 06:15'!
sample
	| urn ballot rand |
	urn := VOUrn new.
	rand := SecureRandom new.
	20
		timesRepeat: [ 
			ballot := VOBallot numberOfPossibleVotes: 5.
			ballot give: 1 additionalPointsTo: (rand nextFrom: 1 to: 5).
			urn addBallot: ballot ].
	^ VOTreeCounting newFrom: urn numberOfChildren: 2! !
VOPublicBoardAuthorityComponent initialize!
